@@ -27,12 +27,12 @@ mbr_format = Struct(
             FAT16_old = 0x04,
             Extended_DOS = 0x05,
             FAT16 = 0x06,
-            FAT32 = 0x0b,
-            FAT32_LBA = 0x0c,
             NTFS = 0x07,
+            FAT32 = 0x0B,
+            FAT32_LBA = 0x0C,
+            ExtendedWithLBA = 0x0F,
             LINUX_SWAP = 0x82,
             LINUX_NATIVE = 0x83,
-            default = Pass,
         ),
         "ending" / BitStruct(
             "head" / BitsInteger(8),
