@@ -13,8 +13,8 @@ from construct.lib import *
 #===============================================================================
 
 MacAddress = ExprAdapter(Byte[6],
-    encoder = lambda obj,ctx: [int(part, 16) for part in obj.split("-")],
     decoder = lambda obj,ctx: "-".join("%02x" % b for b in obj),
+    encoder = lambda obj,ctx: [int(part, 16) for part in obj.split("-")],
 )
 
 ethernet_header = Struct(
@@ -114,8 +114,8 @@ mtp2_header = BitStruct(
 #===============================================================================
 
 IpAddress = ExprAdapter(Byte[4],
-    encoder = lambda obj,ctx: [int(x) for x in obj.split(".")],
     decoder = lambda obj,ctx: "{0}.{1}.{2}.{3}".format(*obj),
+    encoder = lambda obj,ctx: [int(x) for x in obj.split(".")],
 )
 
 ProtocolEnum = Enum(Int8ub,
@@ -125,13 +125,14 @@ ProtocolEnum = Enum(Int8ub,
 )
 
 ipv4_header = Struct(
-    EmbeddedBitStruct(
-        "version" / Const(Nibble, 4),
+    "header" / BitStruct(
+        "version" / Const(4, Nibble),
         "header_length" / ExprAdapter(Nibble,
             decoder = lambda obj, ctx: obj * 4,
             encoder = lambda obj, ctx: obj // 4,
         ),
     ),
+    "header_length" / Computed(this.header.header_length),
     "tos" / BitStruct(
         "precedence" / BitsInteger(3),
         "minimize_delay" / Flag,
@@ -143,7 +144,7 @@ ipv4_header = Struct(
     "total_length" / Int16ub,
     "payload_length" / Computed(this.total_length - this.header_length),
     "identification" / Int16ub,
-    EmbeddedBitStruct(
+    "flags" / BitStruct(
         Padding(1),
         "dont_fragment" / Flag,
         "more_fragments" / Flag,
@@ -167,12 +168,12 @@ ProtocolEnum = Enum(Int8ub,
 )
 
 Ipv6Address = ExprAdapter(Byte[16],
-    encoder = lambda obj,ctx: [int(part, 16) for part in obj.split(":")],
     decoder = lambda obj,ctx: ":".join("%02x" % b for b in obj),
+    encoder = lambda obj,ctx: [int(part, 16) for part in obj.split(":")],
 )
 
 ipv6_header = Struct(
-    EmbeddedBitStruct(
+    "header" / BitStruct(
         "version" / OneOf(BitsInteger(4), [6]),
         "traffic_class" / BitsInteger(8),
         "flow_label" / BitsInteger(20),
@@ -264,7 +265,7 @@ icmp_header = Struct(
             "Echo_request" : echo_payload,
             "Destination_unreachable" : dest_unreachable_payload,
         },
-        default = Pass,
+        # default = Pass,
     ),
 )
 
@@ -562,7 +563,7 @@ tcp_header = Struct(
     "destination" / Int16ub,
     "seq" / Int32ub,
     "ack" / Int32ub,
-    EmbeddedBitStruct(
+    "header" / BitStruct(
         "header_length" / ExprAdapter(Nibble,
             encoder = lambda obj,ctx: obj // 4,
             decoder = lambda obj,ctx: obj * 4,
@@ -581,6 +582,7 @@ tcp_header = Struct(
             "fin" / Flag,
         ),
     ),
+    "header_length" / Computed(this.header.header_length),
     "window" / Int16ub,
     "checksum" / Int16ub,
     "urgent" / Int16ub,
@@ -609,15 +611,15 @@ udp_header = Struct(
 #===============================================================================
 
 class DnsStringAdapter(Adapter):
-    def _decode(self, obj, context):
-        return ".".join(obj[:-1])
-    def _encode(self, obj, context):
-        return obj.split(".") + [""]
+    def _decode(self, obj, context, path):
+        return u".".join(obj[:-1])
+    def _encode(self, obj, context, path):
+        return obj.split(u".") + [u""]
 
 class DnsNamesAdapter(Adapter):
-    def _decode(self, obj, context):
+    def _decode(self, obj, context, path):
         return [x.label if x.islabel else x.pointer & 0x3fff for x in obj]
-    def _encode(self, obj, context):
+    def _encode(self, obj, context, path):
         return [dict(ispointer=1,pointer=x|0xc000) if isinstance(x,int) else dict(islabel=1,label=x) for x in obj]
 
 dns_record_class = Enum(Int16ub,
@@ -732,7 +734,7 @@ layer3_payload = Switch(this.header.protocol,
         "UDP" : layer4_udp,
         "ICMP" : icmp_header,
     },
-    default = Pass,
+    # default = Pass,
 )
 
 layer3_ipv4 = Struct(
@@ -752,7 +754,7 @@ layer2_ethernet = Struct(
             "IPv4" : layer3_ipv4,
             "IPv6" : layer3_ipv6,
         },
-        default = Pass,
+        # default = Pass,
     ),
 )
 
